"use strict";

var ENV = ENV || new Object(null);

ENV.VERSION = /\/tv\/([\d\.]+)\//.test(location.pathname) ?
    location.pathname.match(/\/tv\/([\d\.]+)\//)[1] : '1.0';
ENV.IS_DEV = location.host.search('-dev') !== -1;

// ENV.IS_DEV is rewitten for Samsung in detectPlatform()
ENV.PLATFORM_NAME = detectPlatform();

ENV.LOGGING = false; // TODO: temporary for testing - see App.log
ENV.NO_CACHE = false; // clear=clear in request
ENV.HOST = (ENV.IS_DEV ? 'http://vadim-dev' : 'http://www') + '.kinopoisk.ru';
ENV.IMAGE_FOLDER = ENV.IS_DEV ?
    'http://vadim-dev.kinopoisk.ru/images2/tv_widget/' :
    'http://st.kinopoisk.ru/images/tv_widget/';
ENV.TEMPLATE_DIR = ENV.HOST + '/api/tv/' + ENV.VERSION + '/core/js/templates/';
ENV.REQUEST_HANDLER = ENV.HOST + '/api/tv/' + ENV.VERSION + '/handler_tv.php';

ENV.USER_AGENT = window.navigator ? navigator.userAgent : undefined;

// Ember-related
ENV.RAISE_ON_DEPRECATION = ENV.IS_DEV;

ENV.KEYBOARD = {
    LAT: 'Q~~AZ~~ ~~~WERTYUIOP~~SDFGHJKL~~XCVBNM~~~',
    RUS: 'Й~~ФЯ~~ ~~~ЦУКЕНГШЩЗХЪЫВАПРОЛДЖЭЧСМИТЬБЮ~',
    NUM: '1~~~~~~~~~~234567890~~~~~~~~~~~~~~~~~~~~~'
}

ENV.LANG = {
    RU: {
        LOADING: 'Загрузка',
        LOADING_TRAILER: 'Загрузка трейлера...',
        ON_TODAY: 'На сегодня',
        ON_TOMORROW: 'На завтра',
        ON_WEEKDAY: [ 'На воскресенье', 'На понедельник', 'На вторник', 'На среду',
            'На четверг', 'На пятницу', 'На субботу' ],

        TODAY: 'Сегодня',
        TOMORROW: 'Завтра',
        WEEKDAY: [ 'Воскресенье', 'Понедельник', 'Вторник', 'Среда',
            'Четверг', 'Пятница', 'Суббота' ],
        MONTHS: [ '', 'января', 'февраля', 'марта', 'апреля', 'мая', 'июня',
            'июля', 'августа', 'сентября', 'октября', 'ноября', 'декабря' ]
    }
};

// Key codes

ENV.KEY = {
    UP: 38,
    DOWN: 40,
    LEFT: 37,
    RIGHT: 39,
    ENTER: 13,
    PAGE_UP: 33,
    PAGE_DOWN: 34,
    '0': 48,
    '1': 49,
    '2': 50,
    '3': 51,
    '4': 52,
    '5': 53,
    '6': 54,
    '7': 55,
    '8': 56,
    '9': 57,

    // For keyboard
    CLEAR: 1000001,
    LAT: 1000002,
    RUS: 1000003,
    NUM: 1000004,
    SEARCH: 1000005,
    BACKSPACE: 1000006,
};


/**
 *  WebKit does not support bind
 **/
if (!Function.prototype.bind) {
    Function.prototype.bind = function (oThis) {
        if (typeof this !== "function") {
            // closest thing possible to the ECMAScript 5 internal IsCallable function
            throw new TypeError("Function.prototype.bind - what is trying to be bound is not callable");
        }

        var aArgs = Array.prototype.slice.call(arguments, 1),
            fToBind = this,
            fNOP = function () {},
            fBound = function () {
                return fToBind.apply(this instanceof fNOP ? this : oThis,
                    aArgs.concat(Array.prototype.slice.call(arguments)));
            };

        fNOP.prototype = this.prototype;
        fBound.prototype = new fNOP();

        return fBound;
    };
};

if (!document.querySelector) {
    document.querySelector = function (query) { return $(query).get(0); }
    document.querySelectorAll = function (query) { return $(query); }
}

/**
 *  Dynamically add script file
 **/
function loadScript (filename) {
    if (filename.search('.js') !== -1) {
        var fileref = document.createElement('script');
        fileref.setAttribute('type', 'text/javascript');
        fileref.setAttribute('src', filename);
    } else if (filename.search('.css') !== -1) {
        var fileref = document.createElement('link');
        fileref.setAttribute('rel', 'stylesheet');
        fileref.setAttribute('type', 'text/css');
        fileref.setAttribute('href', filename);
    }

    if (typeof fileref !== 'undefined') {
        $('head').get(0).appendChild(fileref);
    }
}

/**
 *  Feature-detect platform
 **/
function detectPlatform () {
    if (typeof window.NetCastExit === 'function') {
        return 'lg';
    } else if (window.navigator && navigator.userAgent.match(/(SmartHub|Maple\ 6\.0)/i)) {
        ENV.IS_DEV = false; // (window.location.search && window.location.search.search('server=development') !== -1) || false;
        if (!navigator.userAgent.match(/SmartHub/i)) {
            ENV.OLD = true;
        }
        return 'samsung';
    } else if (!!(window.opera && window.opera.version)) {
        return 'opera';
    } else {
        return 'lg';
    }
}
