"use strict";

String.prototype.myCharCodeAt = function (index) {
    return this.charCodeAt(index) !== '~'.charCodeAt(0) ? this.charCodeAt(index) : null;
}

App.Tree.KeyboardNode = App.Tree.ItemNode.extend({
    selector: '.keyboard',
    language: null,
    view: null,
    cursorPosition: 0,
    setShown: function () {
        if (this.get('view')) {
            this.get('view').set('shown', App.SearchRulerController.get('shown') && !App.DetailsView.get('shown'));
        }
    }.observes('App.SearchRulerController.shown', 'App.DetailsView.shown'),
    focusedChanged: function () { // fckin Em.Binding doesnt work
        this.get('view').set('focused', this.get('focused'));
    }.observes('focused'),
    buttonData: function (key, value) {
        if (arguments.length === 1) {
            if (!this.get('_allButtonData')) {
                var allButtonData = {};
                for (var id in ENV.KEYBOARD) {
                    var vocabulary = ENV.KEYBOARD[id],
                        coords = [
                            [ 223, 1, '', vocabulary.myCharCodeAt(0), { left: 1, right: 11, down: 3 } ], // Q [0]
                            [ 103, 1, 'long_96', ENV.KEY.CLEAR, { right: 0, down: 5 } ], // Clear [1]
                            [ 1081, 1, 'long_96', ENV.KEY.BACKSPACE, { left: 21, down: 40 } ], // Backspace [2]
                            [ 233, 71, '', vocabulary.myCharCodeAt(3), { right: 22, up: 0, down: 4 } ], // A [3]
                            [ 243, 141, '', vocabulary.myCharCodeAt(4), { right: 32, up: 3, down: 6 } ], // Z [4]
                            [ 103, 211, '', ENV.KEY.LEFT, { right: 6, up: 1 } ], // <- [5]
                            [ 173, 211, '', ENV.KEY.RIGHT, { left: 5, right: 7, up: 4 } ], // -> [6]
                            [ 383, 211, 'long_344', vocabulary.myCharCodeAt(7), { left: 6, right: 8, up: 35 } ], // Space [7]
                            [ 733, 211, 'long_96', ENV.KEY.LAT, { left: 7, right: 9, up: 38 } ], // LAT [8]
                            [ 835, 211, 'long_96', ENV.KEY.RUS, { left: 8, right: 10, up: 39 } ], // RUS [9]
                            [ 937, 211, 'long_96', ENV.KEY.NUM, { left: 9, up: 40 } ], // 123 [10]
                        ],
                        i;

                    for (i = 1; i <= 11; i += 1) { // [11] .. [21]
                        coords.push([
                            coords[0][0] + i * (6 + 64),
                            coords[0][1],
                            '',
                            vocabulary.myCharCodeAt(i + 10),
                            {
                                left: (i > 1) ? i + 10 - 1 : 0,
                                right: (i < 11) ? i + 10 + 1 : 2,
                                down: (i < 11) ? i + 10 + 11 : 40
                            }
                        ]);
                    }

                    for (i = 1; i <= 10; i += 1) { // [22] .. [31]
                        coords.push([
                            coords[3][0] + i * (6 + 64),
                            coords[3][1],
                            '',
                            vocabulary.myCharCodeAt(i + 21),
                            {
                                left: (i > 1) ? i + 21 - 1 : 3,
                                right: (i < 10) ? i + 21 + 1 : 40,
                                up: i + 21 - 11,
                                down: (i < 10) ? i + 21 + 10 : i + 1
                            }
                        ]);
                    }

                    for (i = 1; i <= 8; i += 1) { // [32] .. [39]
                        coords.push([
                            coords[4][0] + i * (6 + 64),
                            coords[4][1],
                            '',
                            vocabulary.myCharCodeAt(i + 31),
                            {
                                left: (i > 1) ? i + 31 - 1 : 4,
                                right: (i < 8) ? i + 31 + 1 : 40,
                                up: i + 31 - 10,
                                down: (i < 8) ? 7 : i
                            }
                        ]);
                    }

                    // Search
                    coords.push([ 1003, 71, 'long_134', ENV.KEY.SEARCH, { up: 2, left: 31, down: 10 } ]); // [40]

                    allButtonData[id] = coords;
                }
                this.set('_allButtonData', allButtonData);
            }
            return this.get('_allButtonData');
        } else {
            //
        }
    }.property(),
    _allButtonData: null,
    languageChanged: function () {
        var buttonsContainer = $('#keyboard-buttons'),
        coords = this.get('buttonData')[this.get('language')];

        $('.button').removeClass('blank');

        for (var i = 0; i < coords.length; i += 1) {
            var curButton,
                isNew = false;
            if (buttonsContainer.find('.button:nth-child(' + (i + 1) + ')').length) {
                curButton = buttonsContainer.find('.button:nth-child(' + (i + 1) + ')');
            } else {
                curButton = $('<div class="button"></div>');
                isNew = true;
            }

            curButton.css({
                left: coords[i][0],
                top: coords[i][1],
                backgroundPosition: (-coords[i][0]) + 'px ' + (-coords[i][1]) + 'px'
            }).data({
                keycode: coords[i][3],
                directions: coords[i][4]
            });

            if (coords[i][2]) {
                curButton.addClass(coords[i][2]);
            }

            if (coords[i][3] === null) {
                curButton.addClass('blank');
            }

            if (isNew) {
                curButton.appendTo(buttonsContainer);
            }
        }

        this.get('view').set('language', this.get('language'));
    }.observes('language'),
    cursorPositionChanged: function () {
        var fakeInput = $('#fake-input');

        fakeInput.html(this.get('view').get('text').substr(0, this.get('cursorPosition')).replace(/\ /g, '&nbsp;'));
        $('#search-input div').css('backgroundPosition', fakeInput.innerWidth() + 'px 0px'); // TODO: replace with binding
    }.observes('cursorPosition'),
    keyDown: function (key) {
        var success = false,
            currentButton = $('.button.active'),
            tries = 0,
            allDirections = {},
            realActiveButton = null; // Samsung dftIdKey=1319061034&appId=111399000037

        allDirections[ENV.KEY.LEFT] = 'left';
        allDirections[ENV.KEY.RIGHT] = 'right';
        allDirections[ENV.KEY.UP] = 'up';
        allDirections[ENV.KEY.DOWN] = 'down';

        while (!success) {
            var button = $('.button.active'),
                directions = button.data('directions');
            tries += 1;
            if (button.length) {
                var activeButton;
                switch (key) {
                    case ENV.KEY.LEFT:
                    case ENV.KEY.RIGHT:
                    case ENV.KEY.UP:
                    case ENV.KEY.DOWN:
                        if (directions && !isNaN(directions[allDirections[key]])) {
                            activeButton = $($('.button')[directions[allDirections[key]]]);
                            button.removeClass('active');
                            activeButton.addClass('active');
                            if (!activeButton.hasClass('blank')) {
                                success = true;
                            } else {
                                realActiveButton = button;
                            }
                        } else if (key === ENV.KEY.UP && !directions.up &&
                            (directions.left || directions.right || directions.down) // TODO: ??
                        ) {
                            if (realActiveButton) {
                                button.removeClass('active');
                                realActiveButton.addClass('active');
                            }
                            success = true;
                            App.keyboard.up();
                        } else {
                            success = true;
                            $('.button.active').removeClass('active');
                            currentButton.addClass('active');
                        }
                    break;
                    case ENV.KEY.ENTER:
                        button.click();
                        success = true;
                    break;
                    default:
                        //
                    break;
                }
            } else {
                break;
            }
            if (tries >= 10) {
                App.log('loop', currentButton, button, directions); // TODO: test and detect and remove loops
                break;
            }

            if (success && key != ENV.KEY.ENTER) {
                App.SearchRulerController.get('model').autocomplete({ target: $('#search-input div') });
            }
        }
    },
    parent: function (key, value) {
        if (App.tree.get('current') !== this) {
            return null;
        }

        if (arguments.length == 1) {
            if (App.SearchRulerController.get('children') && App.SearchRulerController.get('activeChild') !== this) {
                return App.SearchRulerController.get('activeChild');
            } else {
                return App.SearchRulerController;
            }
            // return this.get('_realParent');
        } else {
            this.set('_realParent', value); // TODO: pointless?
        }
    }.property().volatile(),
    _realParent: null,
    children: function () {
        return null;
    }.property()
});

App.KeyboardViewClass = Em.View.extend({
    language: '',
    text: '',
    classNames: ['keyboard'],
    focused: false,
    classNameBindings: ['shown', 'language', 'focused:focus'],
    shown: false,
    willInsertElement: function () {
        App.Preloader.add([
            'http://st.kinopoisk.ru/images/api_tv/keyboard-blank.png',
            'http://st.kinopoisk.ru/images/api_tv/keyboard-rus.png',
            'http://st.kinopoisk.ru/images/api_tv/keyboard-num.png',
            'http://st.kinopoisk.ru/images/api_tv/keyboard-lat.png',
            'http://st.kinopoisk.ru/images/api_tv/keyboard-cursor.gif'
        ]);
    },
    textChanged: function () {
        $('#search-input div').html(this.get('text').replace(/\ /g, '&nbsp;'));
        App.SearchRulerController.get('model').autocomplete({ target: $('#search-input div') }); // TODO: change model!
    }.observes('text'),
    didInsertElement: function () {
        var view = this;

        App.keyboard = App.Tree.KeyboardNode.create({
            view: this
        });
        App.keyboard.set('language', 'RUS');

        App.SearchRulerController.set('keyboard', App.keyboard);

        $('.button:nth-child(4)').addClass('active'); // 'A' - active

        var $buttons = $('.button');

        view.$().on('click', '.button', function (event) {
            if (typeof event.stopPropagation === 'function') {
                event.stopPropagation();
            }

            App.keyboard.focus();

            var text = view.get('text'),
                cursorPosition = App.keyboard.get('cursorPosition');

            $('.button.active').removeClass('active');
            $(this).addClass('active');
            switch ($(this).data('keycode')) {
                case ENV.KEY.BACKSPACE:
                    if (text && cursorPosition) {
                        view.set('text', text.substr(0, cursorPosition - 1) + text.substr(cursorPosition));
                        App.keyboard.set('cursorPosition', cursorPosition - 1);
                        cursorPosition -= 1;
                    }
                break;
                case ENV.KEY.CLEAR:
                    view.set('text', '');
                    App.keyboard.set('cursorPosition', 0);
                    cursorPosition = 0;
                break;
                case ENV.KEY.LAT:
                case ENV.KEY.RUS:
                case ENV.KEY.NUM:
                    if ($(this).data('keycode') != ENV.KEY[App.keyboard.get('language')]) {
                        for (var id in ENV.KEY) {
                            if (ENV.KEY[id] == $(this).data('keycode')) {
                                App.keyboard.set('language', id);
                                break;
                            }
                        }
                    }
                break;
                case ENV.KEY.SEARCH:
                    App.SearchRulerController.get('model').request();
                break;
                case ENV.KEY.LEFT:
                    cursorPosition = Math.max(0, cursorPosition - 1);
                    App.keyboard.set('cursorPosition', cursorPosition);
                break;
                case ENV.KEY.RIGHT:
                    cursorPosition = Math.min(text.length, cursorPosition + 1);
                    App.keyboard.set('cursorPosition', cursorPosition);
                break;
                default:
                    if (view.get('text').length <= 35) {
                        var arrText = text.split('');
                        arrText.splice(cursorPosition, 0, String.fromCharCode($(this).data('keycode'))); // TODO: substring?
                        view.set('text', arrText.join(''));
                        cursorPosition += 1;
                        App.keyboard.set('cursorPosition', cursorPosition);
                    }
                break;
            }

            // App.SearchRulerController.get('model').autocomplete({ target: $('#search-input div') })
        });
    }
});
