"use strict";

var App = Em.Application.create();

// Useful for static class binding in handlebars
App.True = true;
App.False = false;

App.MainMenuLength = 4; // to properly switch menu items 'cause there're more siblings there
App.MaxTVListPrograms = 11; // all other elements will be hidden, according to focus

App.log = function (server) {
    if (server === true) {
        var newArguments = Array.prototype.slice.call(arguments, 1);
        if (newArguments) {
            $.post(ENV.REQUEST_HANDLER, {
                debug: true,
                platform: ENV.PLATFORM_NAME,
                ua: ENV.USER_AGENT,
                text: JSON.stringify(Array.prototype.map.call(newArguments, String)),
                is_dev: Number(ENV.IS_DEV)
            });
        }
        return;
    }
    if (ENV.LOGGING) {
        if (App.StatusView && ENV.LOGGING) {
            App.StatusView.set('shown', true);
            var newLine = Array.prototype.map.call(arguments, String).join(' '),
                lines = App.StatusView.get('text') ? App.StatusView.get('text').split('<br />') : [];
            lines.push(newLine);
            App.StatusView.set('text', /*newLine*/ lines.slice((lines.length - 2 > 0) ? lines.length - 2 : 0, 2).join('<br />'));
        }
        if (console) {
            console.log.apply(console, arguments); // for browser
            // console.log(Array.prototype.map.call(arguments, String).join(' '));
        }
    }
};


/**
 * Models
 **/

App.Model = amplify;

// TODO: deal with persistent storage - remove XHR

/**
 * Get content for the ruler items
 * TODO: handle 'no more data' response
 **/
App.Model.FilmList = Em.Object.extend({
    isRequesting: false,
    type: 'all',

    request: function (start, limit, otherOptions, callback) { // TODO: callback is not used
        // if (this.get('isRequesting')) {
        //     return false;
        // }
        this.set('isRequesting', true);

        if (typeof otherOptions === 'function') {
            callback = otherOptions;
            otherOptions = null;
        }

        if (typeof start === 'object' && isNaN(limit) && isNaN(otherOptions)) {
            otherOptions = start;
            start = null;
        }

        var otherOptionsString = [];
        if (ENV.IS_DEV && ENV.NO_CACHE) {
            otherOptionsString.push('clear=clear');
        }
        if (otherOptions) {
            for (var id in otherOptions) {
                otherOptionsString.push(id + '=' + otherOptions[id]);
            }
        }

        start = parseInt(start) || 0;
        limit = parseInt(limit) || 5;
        var type = this.get('type');

        var name = [type, start, limit].join('_');
        // TODO: redefine this in more appropriate place OR check for cache
        App.Model.request.define(name, "ajax", {
            url: ENV.REQUEST_HANDLER +
                '?type=' + type + "&start=" + start + '&limit=' + limit + '&platform=' + ENV.PLATFORM_NAME +
                (otherOptionsString.length ? '&' + otherOptionsString.join('&') : ''),
            dataType: "jsonp",
            type: "GET"
            // cache: "persist"
        });

        // copypaste - **** off.
        App.Model.request({
            resourceId: name,
            success: this.successCallback.bind(this),
            error: this.errorCallback.bind(this)
        });
    },

    successCallback: function (data, status) {
        var type = this.get('type'),
            controller = App[type[0].toUpperCase() + type.substr(1) + 'RulerController'];
        if (data && data[type] && data[type].length) {
            var length = controller.get('children') ? controller.get('children').filter(function (item) { return item.get('data'); }).length : 0;
            controller.set('more', false);
            controller.appendChildren(data[type].map(function (item, i, self) {
                if (item == 'more') {
                    controller.set('more', true);
                    return null;
                }

                var releaseDate = null;

                if (!item ||
                    !(item.rus || item.name || item.name_en) ||
                    (controller.get('children') && controller.get('children').filter(function (node) {
                        return node.get('data').id_film == item.id_film;
                    }).length)
                ) {
                    length -= 1; // TODO: ???
                    return null;
                }

                if (item.view_date) { // 2012-01-30
                    releaseDate = moment(item.view_date);
                    releaseDate = releaseDate.isValid() ?
                        releaseDate.format('D MMMM') :
                        (releaseDate.year() && releaseDate.month()) ?
                            releaseDate.format('MMMM YYYY') :
                            null;
                }

                if (item.rating && parseFloat(item.rating)) {
                    var strRating = (parseInt(item.rating * 10, 10) / 10).toString();
                    if (!strRating.match(/\.\d/)) {
                        strRating += '.0';
                    }
                    item.rating = strRating.replace('.', ',');
                } else {
                    item.rating = null;
                }

                item.title = item.rus || item.name || item.name_en;

                if (!item.no_poster) {
                    item.image = ENV.IMAGE_FOLDER + item.id_film + '_400.jpg';
                } else {
                    item.image = 'http://st.kinopoisk.ru/images/api_tv/noposter.gif';
                }

                var resultNode = App.Tree.RulerItemNode.create({
                    loading: true,
                    // image: ,
                    name: item.rus || item.name || item.name_en,
                    'new': Boolean(parseInt(item['new'], 10)),
                    releaseDate: releaseDate,
                    selector: '#' + type + ' .ruler .item:nth-of-type(' + (length + i + 1) + ')', // TODO: #type is OK?
                    data: item
                });

                if (item.image) {
                    App.Preloader.add(resultNode);
                }

                App.SearchCache.set(resultNode.get(Em.GUID_KEY), resultNode);

                return resultNode;
            }));

            App.MouseNavController.update();

            if (typeof callback === 'function') {
                callback();
            }
        }
        this.set('isRequesting', false);
    },

    errorCallback: function (data, status) {
        App.log(data, status);
        this.set('isRequesting', false);
    }
});

App.Model.TVList = App.Model.FilmList.extend({
    type: 'tv',

    request: function (options) {
        if (!options) {
            options = new Object(null);
        }

        if (!options.tv_day) {
            options.tv_day = moment().format('YYYY-MM-DD');
        }

        this.requestWeekdays();
        this._super(options);
    },
    requestWeekdays: function () {
        if (!App.TVRulerController.get('children') || !App.TVRulerController.get('children').length) {
            var weekdayNames = [
                    ENV.LANG.RU.TODAY,
                    ENV.LANG.RU.TOMORROW,
                    // ENV.LANG.RU.WEEKDAY[moment().add('days', 1).day()],
                    ENV.LANG.RU.WEEKDAY[moment().add('days', 2).day()],
                    ENV.LANG.RU.WEEKDAY[moment().add('days', 3).day()],
                    ENV.LANG.RU.WEEKDAY[moment().add('days', 4).day()],
                    ENV.LANG.RU.WEEKDAY[moment().add('days', 5).day()],
                    ENV.LANG.RU.WEEKDAY[moment().add('days', 6).day()]
                ],
                weekdayCount, weekdayNode, showDate = false;
            for (weekdayCount = 1; weekdayCount <= weekdayNames.length; weekdayCount += 1) {
                weekdayNode = App.Tree.TVWeekdayNode.create({
                    text: showDate ?
                        moment().add('days', weekdayCount).format('D MMMM') :
                        weekdayNames[weekdayCount - 1],
                    day: moment().add('days', weekdayCount - 1).format('YYYY-MM-DD'),
                    selector: '#weekdays > li:nth-of-type(' + weekdayCount + ')'
                }).set('childrenController', Em.ArrayController.create({
                    selector: '#tv .ruler:nth-of-type(' + weekdayCount + ')',
                    _model: this
                }));

                App.TVRulerController.appendChildren(weekdayNode);

                if (weekdayNames[weekdayCount - 1] == ENV.LANG.RU.WEEKDAY[0]) {
                    showDate = true;
                }
            }
        }
    },
    successCallback: function (data, status) {
        var showFirstRuler =
            !Em.get('App.TVRulerController.children').filter(function (item) {
                return item.get('shown');
            }).length;

        App.TVRulerController.set('more', false);

        if (data && data.tv && data.tv.length) {
            var days = {},
                channelNames = [];

            data.tv.forEach(function (item) {
                if (item == 'more') {
                    App.TVRulerController.set('more', true);
                    return null;
                }

                if (!item || !(item.rus || item.name || item.name_en)) {
                    return null;
                }

                item.name = (item.rus || item.alt_name || item.name) +
                    (/*item.note*/false ? (' <span class="gray">(' + item.note + ')</span>') : '');
                item.time = item.seans_t.substr(0, 5);
                if (item.rating && parseFloat(item.rating)) {
                    var strRating = (parseInt(item.rating * 10, 10) / 10).toString();
                    if (!strRating.match(/\.\d/)) {
                        strRating += '.0';
                    }
                    item.rating = strRating.replace('.', ',');
                } else {
                    item.rating = null;
                }

                item.title = item.rus || item.name || item.name_en;

                days[item.day] = days[item.day] || [];

                days[item.day][item.channel_index] = days[item.day][item.channel_index] || [];
                days[item.day][item.channel_index].push(item);

                channelNames[item.channel_index] = channelNames[item.channel_index] || item.tv_name;
            });

            App.log('[TV model] callback', data, days);

            for (var day in days) {
                var weekdayNode = App.TVRulerController.get('children').filter(function (item) {
                    return item.get('day') === day;
                })[0];

                if (!weekdayNode) {
                    App.log('[TV model] no such day:', day);
                    continue;
                }

                // realChannelIndex !== channel_index e.g. where there's no 1TV channel in response
                var realChannelIndex = weekdayNode.get('children') ? weekdayNode.get('children').length : 0;
                days[day].forEach(function (programData, channel_index) {
                    if (!programData.length) {
                        return false;
                    }

                    if (realChannelIndex && weekdayNode.get('children').filter(function (node) {
                        return node.get('text') == channelNames[channel_index];
                    }).length) {
                        App.log('[TV model] duplicate channel:', channelNames[channel_index]);
                        return;
                    }


                    var channelNode = App.Tree.TVChannelNode.create({
                        text: channelNames[channel_index],
                        style: 'background-image: url(http://st.kinopoisk.ru/images/tv/logo_' + programData[0].id_tv + '.gif);',
                        selector: weekdayNode.get('childrenController').get('selector') +
                            ' .channels > li:nth-of-type(' + (realChannelIndex + 1) + ')'
                    });
                    realChannelIndex += 1;

                    var programCount = 0;
                    var programs = programData.map(function (item) {
                        programCount += 1;

                        if (!item.no_poster) {
                            item.image = ENV.IMAGE_FOLDER + item.id_film + '_400.jpg';
                        } else {
                            item.image = 'http://st.kinopoisk.ru/images/api_tv/noposter.gif';
                        }

                        var programNode = App.Tree.TVRulerItemNode.create({
                            hidden: programCount > App.MaxTVListPrograms,
                            selector: channelNode.get('selector') +
                                ' .programs > li:nth-of-type(' + programCount + ')',
                            data: item,
                        });

                        // this can possibly decrease perfomance on long TV Program dives
                        // App.Preloader.add(programNode);
                        App.SearchCache.set(programNode.get(Em.GUID_KEY), programNode);

                        return programNode;
                    });
                    channelNode.appendChildren(programs);
                    // channelNode.set('arrowDownShown', programCount > App.MaxTVListPrograms);
                    weekdayNode.appendChildren(channelNode);

                });
            }

            if (showFirstRuler) {
                App.TVRulerController.get('children').objectAt(0)./*get('children').objectAt(0).*/set('shown', true);
                App.log('[tv model] showFirstRuler', showFirstRuler, !App.TVRulerController.get('children'));
            }
        }
        this.set('isRequesting', false);
    },
});


App.Model.FilmSearch = Em.Object.extend({
    isRequesting: false,
    searchPhrase: '',
    searchPhraseTimeout: null,
    searchHistory: [],
    // queryID: null,
    searchPhraseChanged: function() {
        if (this.get('searchPhraseTimeout')) {
            clearTimeout(this.get('searchPhraseTimeout'));
            this.set('searchPhraseTimeout', null);
        }
        this.set('searchPhraseTimeout',
            setTimeout(this.request.bind(this), this.get('autocompleteSettings').delay));
    }.observes('searchPhrase'),

    autocompleteSettings: {
        delay: 2500,
        minSymbolsNum: 2,
        target: null
    },
    autocomplete: function (newSettings) {
        var settings = $.extend({}, this.get('autocompleteSettings'), newSettings);
        this.set('autocompleteSettings', settings);

        if (!settings.target ||
            !settings.target.length ||
            settings.target.text() === this.get('searchPhrase') ||
            settings.target.text().length < settings.minSymbolsNum
        ) {
            this.set('searchPhraseTimeout', clearTimeout(this.get('searchPhraseTimeout')));
        } else {
            App.log('[_pre_ search go]', this.get('searchPhrase', settings.target.text()));
            this.set('searchPhrase', settings.target.text());
        }
    },

    request: function (childrenArrayLength) {
        var start = childrenArrayLength || 0;

        this.set('searchPhraseTimeout', clearTimeout(this.get('searchPhraseTimeout')));

        if (!this.get('searchPhrase') || !this.get('searchPhrase').length) {
            return false;
        }

        // if (this.get('isRequesting')) {
        //     return false;
        // }
        this.set('isRequesting', true);
        // this.set('queryID', Math.random());
        // TODO: huge mess! why this.get('searchPhrase') & this.get('autocompleteSettings').target.text() differs by 1 character?
        var searchPhrase = this.get('autocompleteSettings').target ?
            this.get('autocompleteSettings').target.text() :
            this.get('searchPhrase');

        var searchHistory = this.get('searchHistory');
        searchHistory.splice(0, 0, searchPhrase + ',' + start);

        if (searchHistory[searchHistory.indexOf(searchPhrase + ',' + start) + 1] == this.get('searchPhrase') + ',' + start) {
            App.log('[search] duplicate search',
                searchPhrase + ',' + start,
                searchHistory.indexOf(searchPhrase + ',' + start),
                this.get('searchPhrase') + ',' + start,
                searchHistory.indexOf(this.get('searchPhrase')));
            this.set('isRequesting', false);
            return false;
        }

        this.set('searchHistory', searchHistory);

        $.getJSON(ENV.REQUEST_HANDLER, {
            search: searchPhrase,
            start: start,
            platform: ENV.PLATFORM_NAME,
        }).done(this.successCallback.bind(this))
            .fail(this.errorCallback.bind(this));

        // copypaste - **** off.
        App.log('[search go]', searchPhrase);
    },

    successCallback: function (rawData, status) {
        App.log('[search result]', rawData, status);
        this.set('isRequesting', false);

        var searchHistory = this.get('searchHistory');

        if (rawData.search_phrase == this.get('searchPhrase') &&
            rawData.start &&
            this.get('children') &&
            rawData.start != this.get('children').length
        ) {
            App.log('[search] invalid offset', rawData.search_phrase, rawData.start, this.get('children').length);
            return false;
        } else if (searchHistory.indexOf(rawData.search_phrase + ',' + rawData.start) > searchHistory.indexOf(this.get('searchPhrase') + ',' + rawData.start)) {
            App.log('[search] late search',
                rawData.search_phrase + ',' + rawData.start,
                searchHistory.indexOf(rawData.search_phrase + ',' + rawData.start),
                this.get('searchPhrase'),
                searchHistory.indexOf(this.get('searchPhrase')));
            return false;
        }

        var data = rawData.data,
            firstNode = null;
        // for (var id in rawData) {
        //     data.push(rawData[id]);
        // }

        App.log('[search] replace children:', !rawData.start);

        if (data && data.length) {
            var length = (rawData.start && App.SearchRulerController.get('children')) ?
                    App.SearchRulerController.get('children').filter(function (item) { return item && item.get('data'); }).length : 0;

            App.SearchRulerController.set('more', false);
            App.SearchRulerController.appendChildren(data.map(function (item, i, self) {
                if (item == 'more') {
                    App.SearchRulerController.set('more', true);
                    return null;
                }

                var releaseDate = null;

                if (!item ||
                    !(item.rus || item.name || item.name_en) ||
                    (rawData.start && App.SearchRulerController.get('children') && App.SearchRulerController.get('children').filter(function (node) {
                        return node && node.get('data').id_film == item.id_film;
                    }).length)
                ) {
                    length -= 1;
                    return null;
                }

                if (item.view_date) {
                    releaseDate = moment(item.view_date);
                    releaseDate = releaseDate.isValid() ?
                        releaseDate.format('D MMMM') :
                        (releaseDate.year() && releaseDate.month()) ?
                            releaseDate.format('MMMM YYYY') :
                            null;
                }

                if (item.rating && parseFloat(item.rating)) { // TODO: do it on server
                    var strRating = (parseInt(item.rating * 10, 10) / 10).toString();
                    if (!strRating.match(/\.\d/)) {
                        strRating += '.0';
                    }
                    item.rating = strRating.replace('.', ',');
                } else {
                    item.rating = null;
                }

                item.image_small = (!item.image || /image_none/.test(item.image)) ? 'http://st.kinopoisk.ru/images/api_tv/noposter.gif' : item.image;
                item.image = (item.image && !item.no_poster) ? item.image.replace('film', 'film_big') : 'http://st.kinopoisk.ru/images/api_tv/noposter.gif';

                var resultNode = App.Tree.RulerItemNode.create({
                    // image: ,
                    selector: '#search .ruler .item:nth-of-type(' + (length + i + 1) + ')',
                    nocache: true,
                    releaseDate: releaseDate,
                    data: item
                });

                if (App.keyboard) { // TODO: not necessary?
                    resultNode.set('activeChild', App.keyboard);
                }

                firstNode = firstNode || resultNode;

                if (item.image) {
                    App.Preloader.add(resultNode);
                }

                return resultNode;
            }), !rawData.start); // replace children
            if (typeof callback === 'function') {
                callback();
            }
        }

        if (!rawData.start) {
            App.SearchRulerController.set('activeChild', firstNode);
        }

        App.MouseNavController.update(); // TODO: find nicer way to display nav arrows

        if (App.SearchRulerController.get('children')) {
            // App.SearchRulerController.get('children').objectAt(0).focus();
        } else {
            // empty result
        }
    },

    errorCallback: function (data, status) {
        App.log('[search result]', data, status);
        this.set('isRequesting', false);
    }

});

App.Model.loadView = function (name, vendorClass, callback) {
    if (typeof callback === 'undefined' && typeof vendorClass === 'function') {
        callback = vendorClass;
        vendorClass = undefined;
    }

    App.Model.request.define(name + '.handlebars', 'ajax', {
        url: ENV.TEMPLATE_DIR + name + '.handlebars' +
            (ENV.NO_CACHE ? '?rnd=' + Math.random() : ''),
        dataType: "text",
        type: "GET"
        // cache: "persist"
    });
    App.Model.request({
        resourceId: name + '.handlebars',
        success: function (data, status) {
            App.containerView = App.containerView || Em.ContainerView.create().append();
            Em.TEMPLATES[name] = Em.Handlebars.compile(data);

            var viewObjectName = name[0].toUpperCase() + name.substr(1) + 'View',
                viewClassName = viewObjectName + 'Class';
            App[viewObjectName] = (vendorClass || App[viewClassName]).create({
                templateName: name,
                list: true // TODO: this is for App.RulerView only
            });

            App.containerView.pushObject(App[viewObjectName]);

            if (typeof callback === 'function') {
                callback(App[viewObjectName]);
            }
        },
        error: function (data, status) {
            App.log(data, status, ENV.TEMPLATE_DIR + name + '.handlebars');
        }
    });
};

App.Model.loadSynopsis = function () {
    var data = App.tree.get('current').get('data');

    if (this.get('shown')) {
        if (data.synopsis) {
            this.set('synopsis', data.synopsis);

            // testSynopsis(data.synopsis, data.id_film);
            // testHeader(data.title, data.id_film);
        } else {
            this.set('synopsis', 'Загрузка...');
            $.get(ENV.REQUEST_HANDLER, {
                platform: ENV.PLATFORM_NAME,
                synopsis: true,
                id_film: data.id_film
            }).always(function (e) { // TODO: starnge = done() not work
                if (e && typeof e.responseText == 'string') {
                    data.synopsis = e.responseText;
                    App.tree.get('current').set('data', data);
                    App.DetailsView.set('synopsis', e.responseText);

                    // testSynopsis(e.responseText, data.id_film);
                    // testHeader(data.title, data.id_film);
                } else {
                    // TODO: handle error
                    App.DetailsView.set('synopsis', null);
                }
            });
        }
    } else {
        this.set('synopsis', ' ');
    }
};


/**
 * [Image] Resource Preloader
 **/

App.Preloader = {
    container: null,
    // [ { '/path/to/image.jpg': itemNodeObject }, ... ]
    query: [],
    add: function (elements) {
        if (!elements) {
            return false;
        }

        if (!App.Preloader.container) {
            App.Preloader.container = $('<div id="preloader"></div>').appendTo('body');
        }

        if (!Em.isArray(elements)) {
            elements = [elements];
        }

        elements.forEach(function (item) {
            if (typeof item === 'string') {
                App.Preloader.pushItem({
                    src: item,
                    node: null,
                    preloaded: false
                });
            } else {
                if (item.get('data').image) {
                    App.Preloader.pushItem({
                        src: item.get('data').image,
                        node: item,
                        preloaded: false
                    });
                }

                if (item.get('data') && item.get('data').back_url) {
                    App.Preloader.pushItem({
                        src: item.get('data').back_url,
                        node: null,
                        preloaded: false
                    });
                }
            }
        });
    },
    pushItem: function (outerItem) {
        return (function (item) {
            App.Preloader.query.push(item); // ??
            var src = item.src;
            item.preloaded = true;

            var timeout;
            var $img = $('<img src="' + item.src + '" />')
                .appendTo(App.Preloader.container)
                .load(function () {
                    if (!this.width) {
                        App.log('[Preloader] no image with url', this.src);
                        return;
                    }

                    clearTimeout(timeout);
                    App.Preloader.updateNode(item, this);
                    // App.Preloader.query.splice(index, 1);
                    // App.log('[Preloader] OK', this.src, this.width, item);
                    $(this).remove();
                });

            timeout = setTimeout(function () {
                App.Preloader.updateNode(item, null);
                // App.Preloader.query.splice(index, 1);
                App.log('[Preloader] fail', item.src);
            }, 2000);
        }) (outerItem);
    },
    updateNode: function (item, imageData) {
        return (function () {
            if (item.node) {
                var interval = setInterval(function () {
                    // App.log('[Preloader] onload', imageData, imageData ? imageData.src : 'no src');
                    if (!item.node.$().length) {
                        App.log('interval', item);
                        return;
                    }
                    clearInterval(interval);
                    if (/(tv|search)/.test(item.node.get('selector'))) {
                        return;
                    }
                    imageData = imageData || { src: 'http://st.kinopoisk.ru/images/api_tv/noposter.gif', clientWidth: 270 };
                    var oldBackground = item.node.$().css('backgroundImage').split(',')[0] || 'none'; // 'New' bagde?
                    var newBackground = oldBackground + ', url(' + imageData.src + ')';
                    item.node.set('loading', false);

                    item.node.set('style', 'background-image: ' + newBackground);
                }, 200);
            }
        } ());
    }
};


/**
 * Search Cache
 * TODO: potential bug with search result items. And use this: http://d.pr/i/FGVZ
 **/

App.SearchCache = {
    cache: new Object(null),
    get: function (key) {
        if (!key) {
            // App.log('[Search Cache] no key to get');
            return null;
        }
        if (App.SearchCache.cache.hasOwnProperty(key)) {
            return App.SearchCache.cache[key];
        } else {
            // App.log('[Search Cache] no cache for key', key);
            return null;
        }
    },
    set: function (key, value) {
        // if (!nodesOrKey) {
        //     return false;
        // }

        // if (nodesOrKey.each) {
        //     nodesOrKey.each(function (node) {
        //         App.SearchCache.set(node.get(Em.GUID_KEY), node);
        //     });
        // } else if (nodesOrKey.forEach) {
        //     nodesOrKey.forEach(function (node) {
        //         if (node.each) {
        //             node.each(function (innerNode) {
        //                 App.SearchCache.set(innerNode.get(Em.GUID_KEY), innerNode);
        //             });
        //         }
        //         // App.SearchCache.set(node.get(Em.GUID_KEY), node);
        //     });
        // } else {
            if (!key || (Em.Object.detectInstance(value) && value.get('nocache'))) {
                // App.log('[Search Cache] no key to set');
                return false;
            }
            if (App.SearchCache.cache.hasOwnProperty(key)) {
                // App.log('[Search Cache] key rewrite', key, value);
            }

            App.SearchCache.cache[key.toString()] = value;
        // }
    },
};


/**
 * Background Controller
 * TODO: more DOM manipulation. Refactor this to View Controller
 **/

App.Background = {
    defaultURI: null,
    change: function (uri) {
/*        uri = uri ? (ENV.HOST + uri) : App.Background.defaultURI;
        var oldURI = $('body').css('backgroundImage');
        if (oldURI !== 'url(' + uri + ')' && oldURI !== uri) {
            App.log('[App.Background]', uri);
            $('body').css('backgroundImage', uri ? 'url(' + uri + ')' : 'none');
        }
*/    },
    remove: function () {
        App.Background.change();
    }
};


/**
 * Controllers
 **/

// Incapsulation - hell yeah
// TODO: This 'proxifying' is tricky and messy. Remove!
// Check ObjectProxy and ObjectController

// TODO: refactor activeChild - should be recursive to the top
// TODO: this is the right way: http://matchingnotes.com/ember-array-proxy/filterable-mixin
Em.ArrayController.reopenClass({
    _model: null,
    parent: null,
    // proxy to 'content' - ****IN' stupid!
    children: function (key, value) {
        if (arguments.length === 1) {
            return this.get('content');
        } else {
            this.set('content', value);
        }
    }.property('content'),
    activeChild: function (key, value) {
        if (arguments.length === 1) {
            if (this.get('parent').get('activeChild') === null) {
                // if (this.get('children') && this.get('children').length) {
                //     this.set('activeChild', this.get('children')[0]);
                //     this.get('activeChild').focus();
                // }
                this.get('parent').set('activeChild', this.get('content').objectAt(0));
            }
            return this.get('parent').get('activeChild');
        } else {
            if (this.indexOf(value) === -1) {
                this.addObject(value);
            }
            this.get('parent').set('activeChild', value);
        }
    }.property().volatile(), // TODO: read more about .volatile()

    selector: '',
    $: function () { // TODO: need to incapsulate this from App.Tree.ItemNode
        return $(this.get('selector'));
    },

    focus: function () {
        if (this.get('parent')) {
            return this.get('parent').focus();
        }
    }
});

App.Tree = new Object(null);

App.Tree.ItemNode = Em.Object.extend({
    loading: false,
    hidden: false,
    id: null,
    parent: null,
    data: new Object(null),
    realActiveChild: null,
    style: null,
    activeChild: function (key, value) {
        // TODO: move this to App.SearchRulerController
        if ((this === App.SearchRulerController ||
            this.get('parent') === App.SearchRulerController) &&
            App.keyboard &&
            arguments.length == 1 &&
            !this.get('children').length
        ) {
            return App.keyboard;
        } else {
            if (arguments.length === 1) {
                if (!this.get('realActiveChild')/* === null*/ &&
                    this.get('children') &&
                    this.get('children').objectAt(0) !== App.keyboard
                ) {
                    this.set('realActiveChild', this.get('children').objectAt(0));
                }
                return this.get('realActiveChild');
            } else {
                if (!this.get('children') || this.get('children').indexOf(value) === -1) {
                    this.appendChildren(value);
                }

                this.set('realActiveChild', value);
            }
        }
    }.property(/*'parent', 'children', 'realActiveChild'*/).volatile(),
    isActive: function () {
        var activeChild = this.get('parent') ?
            this.get('parent').get('activeChild') : null;
        return this === activeChild;
    }.property().volatile(),
    children: function (key, value) {
        if (arguments.length === 1) {
            return this.get('_childrenContainer');
        } else {
            if (Em.isArray(value)/*value.forEach*/) {
                if (!value.length) {
                    value = [value];
                }
                this.set('_childrenContainer', value);
                this.get('_childrenContainer').forEach(function (item) {
                    if (!Em.Object.detectInstance(item)) {
                        item = Em.Object.create(item); // TODO: check if item s Object
                    }
                    item.set('parent', this);
                }, this);
                return this._childrenContainer;
            } else {
                this._childrenContainer = [];
                return null;
            }
        }
    }.property('_childrenContainer', 'parent'),
    appendChildren: function (value) { // copy-paste from RulerNode or something like this
        if (!Em.isArray(value)) {
            value = [value];
        }

        if (this.get('_childrenContainer') === null) {
            this.set('children', value);
            return true;
        }

        value.forEach(function (item) {
            if (!Em.Object.detectInstance(item)) {
                item = Em.Object.create(item); // TODO: check if item s Object
            }

            item.set('parent', this);
            // item.set('parent', this.get('_childrenContainer'));
        }, this);
        if (this.get('_childrenContainer') !== null) {
            var content = this.get('_childrenContainer');
            Array.prototype.push.apply(content, value);
            this.set('_childrenContainer', content);
        } else {
            this.set('_childrenContainer', value);
        }

    },

    _childrenContainer: [],
    selector: null,
    siblings: Em.computed(function (key, value) { // this - excluded
        // getter
        if (arguments.length === 1) {
            var siblings = this.get('parent') ?
                this.get('parent').get('children').filter(function (item) {
                    return item !== this;
                }, this) : [];
            if (App.MainMenuLength && this.get('parent') === App.tree) {
                siblings = siblings.slice(0, App.MainMenuLength - 1);
            }
            return siblings;
        // setter
        }
    }).property('parent'),

    allSiblings: Em.computed(function (key, value) { // this - included
        // getter
        if (arguments.length === 1) {
            if (App.MainMenuLength && this.get('parent') === App.tree) {
                return App.tree.get('children').slice(0, App.MainMenuLength);
            } else {
                return this.get('parent') ? this.get('parent').get('children') : [];
            }
        // setter
        }
    }).property('parent'),

    $: function () {
        return $(this.get('selector'));
    },

    focused: function () {
        return this === Em.get('App.tree.current');
    }.property('App.tree.current'),// Em.computed.equal(this, 'App.tree.current'),

    focus: function (node) {
        node = node || this;
        if (!App.tree) { // we will not use return vals
            return;
        }

        if (node.get('data')) {
            App.Background.change(node.get('data').back_url);
        }

        if (!App.tree.get('current')) {
            App.tree.set('current', node);
        } else if (!node.get('focused')) {
            App.tree.set('previous', App.tree.get('current'));
            App.tree.set('current', node);

            // to sync focus changes
            // TODO: now .focus is binded to 'node.focused' - remove dat binding
            Em.run.scheduleOnce('afterRender', this, function (prev, cur) {
                prev.$().removeClass('focus');
                cur.$().addClass('focus');
            }, Em.get('App.tree.previous'), node);
        }

        try {
            App.tree.get('current').get('parent').set('activeChild', App.tree.get('current'));
        } catch (e) {
            App.log('[ItemNode.focus] no parent for ', App.tree.get('current'));
        }
    },

    up: function () { // TODO: assume that this === this.get('current') - is needed for child classes, maybe
        var currentParent = App.tree.get('current.parent');
        if (currentParent && currentParent !== App.tree) {
            currentParent.set('activeChild', App.tree.get('current'));
            currentParent.focus();
        }
    },

    down: function () {
        if (this.get('activeChild')) {
            this.get('activeChild').focus();
        } else {
            return false;
        }
    },

    find: function (element) {
        if (!element.dataset) {
            element.dataset = [];
        }

        if (element.id && App.SearchCache.get(element.id)) {
            element.dataset['cacheId'] = element.id;
            return App.SearchCache.get(element.dataset['cacheId']);
            App.log('[find] got cache for', element.dataset['cacheId']);
        } else if (element.dataset['cacheId'] && App.SearchCache.get(element.dataset['cacheId'])) {
            return App.SearchCache.get(element.dataset['cacheId']);
            App.log('[find] got cache for', element.dataset['cacheId']);
        }

        var found = null;
        this.each(function (node) {
            if (found) {
                return;
            }
            // App.log('[find]', node.$().get(0), element);
            if (node.$().get(0) === element || node.$().find(element).length) {
                found = node;
                if (!element.dataset['cacheId']) {
                    element.dataset['cacheId'] = node.get(Em.GUID_KEY);
                }

                App.SearchCache.set(element.dataset['cacheId'], node); // TODO: set() has to be done once, not countless times!
            }

            // App.SearchCache.set(node.get('cacheId'), node);
        });
        return found;
    },

    /**
     * Each function for tree
     * Goes from children up to parents
     **/
    each: function (callback) {
        if (typeof callback === 'function') {
            if (this.get('children') &&
                this.get('children').length &&
                this.get('children').objectAt(0) !== App.keyboard
            ) {
                // App.log('[each]', this, 'children', this.get('children'));
                this.get('children').forEach(function (node) {
                    if (node && typeof node.each === 'function') {
                        node.each(callback);
                    } else if (!node) {
                        App.log('[each] null item', node, this);
                    }
                });
            }

            callback(this);
        }
    },

    index: function () { // node's index in siblings array
        var siblings = this.get('parent').get('children') || this.get('parent');
        if (siblings) { // TODO: maybe filterProperty will work?
            var index = null, counter = 0;
            siblings.forEach(function (item) {
                if (item === this) {
                    index = counter;
                }
                counter += 1;
            }, this);
            return index;
        } else {
            return null;
        }
    }

});

App.Tree.RulerItemNode = App.Tree.ItemNode.extend({
    invertedKeyNavigation: false,
    jumpFromLastToFirst: false, // false for 'infinite-scroll' nodes (e.g not TV) TODO: working??
    // TODO: use this._super
    keyDown: function (key) {
        var index = this.index(),
            siblings = this.get('allSiblings'),
            length = siblings.length;

        if (this.get('invertedKeyNavigation') &&
            [ENV.KEY.LEFT, ENV.KEY.RIGHT].indexOf(key) !== -1
        ) {
            this.up();
            App.tree.get('current').keyDown(key);
            App.tree.get('current').down();
            return;
        } else if (this.get('invertedKeyNavigation') &&
            !(!this.get('parent').get('invertedKeyNavigation') && // TODO: always true??
                index === 0 && key === ENV.KEY.UP)
        ) {
            if (key === ENV.KEY.RIGHT) {
                key = ENV.KEY.DOWN;
            } else if (key === ENV.KEY.LEFT) {
                key = ENV.KEY.UP;
            } else if (key === ENV.KEY.UP) {
                key = ENV.KEY.LEFT;
            } else if (key === ENV.KEY.DOWN) {
                key = ENV.KEY.RIGHT;
            }
        }

        switch (key) {
            case ENV.KEY.RIGHT:
                if (index + 1 < length) {
                    this.focus(siblings[index + 1]);
                }
            break;
            case ENV.KEY.LEFT:
                if (index > 0) {
                    this.focus(siblings[index - 1]); // TODO: rewrite via siblings
                }
            break;
            case ENV.KEY.UP:
                this.up();
            break;
            case ENV.KEY.DOWN:
                this.down();
            break;
            case ENV.KEY.ENTER:
                this.focus();
            break;
            default:
            break;
        }

        var current = App.tree.get('current');
        if (current.get('invertedKeyNavigation')) {
            var siblings = current.get('siblings'),
                minVisible = Math.max(0, current.index() - Math.floor(App.MaxTVListPrograms / 2 - 1)),
                maxVisible = minVisible + App.MaxTVListPrograms - 1;
                // arrowUpShown = false,
                // arrowDownShown = false;

            for (var i = 0, _l = siblings.length; i < _l; i += 1) {
                if (_l > App.MaxTVListPrograms &&
                    (i < minVisible || i >= maxVisible) // TODO: why >= ?
                ) {
                    // siblings[i].$().css('display', 'none');
                    siblings.objectAt(i).set('hidden', true);
                    // if (i === 0) {
                    //     arrowUpShown = true;
                    // } else if (i === _l - 1) {
                    //     arrowDownShown = true;
                    // }
                } else {
                    siblings.objectAt(i).set('hidden', false);
                    // siblings[i].$().css('display', 'block');
                }
            }
            // this.get('parent').set('arrowUpShown', arrowUpShown)
            //                   .set('arrowDownShown', arrowDownShown);
            App.MouseNavController.update();
        }

        // App.log('[RulerItemNode.keydown]', key, 'index', index, 'length', length);
    },
    focus: function (node) {
        node = node || this;
        if (!App.tree) {
            return false;
        }

        // load additional items to the right
        if (!Em.get('App.tree.current.data.id_tv') &&
            node.get('allSiblings').length - node.index() < 6) {
            node.get('parent').get('model').request(node.get('allSiblings').length);
        }

        if (App.tree.get('current') === node) {
            this._super(node);
            this.get('parent').rulerMove();
            App.DetailsView.set('shown', !App.DetailsView.get('shown'));
        } else {
            App.DetailsView.set('shown', false);

            this._super(node);
            this.get('parent').rulerMove();
        }
    }
});

App.Tree.TVRulerItemNode = App.Tree.RulerItemNode.extend({
    invertedKeyNavigation: true,
    focus: function (node) {
        (node || this).get('parent').get('parent').set('activeChild', (node || this).get('parent')); // TODO: maybe implement in some generic class?
        this._super(node);
    }
});


/**
 *  Controls
 **/

App.Tree.ControlsNode = App.Tree.ItemNode.extend({
    size: 'small',
    focus: function (node) {
        node = node || this;
        if (!App.tree) {
            return;
        }

        if (App.tree.get('current') === node) {
            this.action();
        } else {
            this._super(node);
        }
    },
    action: function () {
        return false;
    }
});


App.Tree.RootNode = App.Tree.ItemNode.extend({
    current: null,
    previous: null,
    parent: function () {
        return this;
    }.property(),
    currentChanged: function () {
/*
        if (this.get('current') !== this && this.get('current').$().length) {

        }
*/
    }.observes('current'),
    down: function () {},
    move: function () {}
});

App.Tree.MenuNode = App.Tree.ItemNode.extend({

});

App.Tree.RulerNode = App.Tree.ItemNode.extend({
    more: true,
    hidden: false,
    model: function() {
        return this.get('childrenController._model');
    }.property('childrenController'),
    childrenController: null,
    childrenControllerChanged: function () {
        this.get('childrenController').set('parent', this);
        // this.get('childrenController').forEach(function(item) {
        //     item.set('parent', this);
        // }, this);
    }.observes('childrenController'),
    children: /*Em.computed(*/function (key, value) {
        if (arguments.length === 1) {
            try {
                return this.get('childrenController') ? this.get('childrenController').get('content') : [];
            } catch (e) {
                App.log('[App.Tree.RulerNode.children]', e);
                return [];
            }
        } else { // this will erase previous children
            if (!Em.isArray(value)) {
                value = [value];
            }

            if (this.get('childrenController') === null) {
                this.set('childrenController', Em.ArrayController.create({ content: value }));
            }

            value.forEach(function (item) {
                if (!Em.Object.detectInstance(item)) { // not needed anymoar??
                    item = Em.Object.create(item); // TODO: check if item is Object
                }
                item.set('parent', this/*.get('childrenController')*/);
            }, this);

            this.get('childrenController').set('content', value);
        }
        /* else { // this will APPEND children

            // This became ****ed up with mathing this.get('children') === undefined => moved to appendChildren

            // if (this.get('childrenController') !== null) {
            //     value.forEach(function(item) {
            //         item.set('parent', this.get('childrenController'));
            //     }, this);
            //     this.get('childrenController').addObjects(value);
            // } else {
            //     this.set('childrenController', Em.ArrayController.create({ content: value })); // TODO: App.RulerArrayController?
            // }
        } */
    }.property(/*'childrenController', 'childrenController.content'*/).volatile(),
    appendChildren: function (value, replace) {
        var childrenController = this.get('childrenController');
        if (!Em.isArray(value)) {
            value = [value];
        }

        if (childrenController === null) {
            this.set('children', value);
            return true;
        }

        var cleanValue = [];
        value.forEach(function (item) {
            if (item && item.set) {
                item.set('parent', this);
                cleanValue.push(item);
            }
        }, this);
        value = cleanValue;

        if (childrenController !== null) {
            if (childrenController.get('content')) {
                if (replace && childrenController.get('content')) {
                    childrenController.replaceContent(0, childrenController.get('content').length, value);
                    childrenController.set('activeChild', value.objectAt(0));
                } else {
                    childrenController.addObjects(value);
                }
            } else {
                childrenController .set('content', value);// addObjects(value);
            }
        } else {
            this.set('childrenController', Em.ArrayController.create({ content: value })); // TODO: App.RulerArrayController?
        }
    },
    focus: function (node) {
        node = node || this;
        if (!App.tree) {
            return false;
        }

        App.DetailsView.set('shown', false);

        this._super(node);

        if (node.get('model') &&
            (!node.get('children') || !node.get('children').length)
        ) {
            var tvOptions;
            if (node.get('day')) {
                tvOptions = { tv_day: node.get('day') };
            }
            node.get('model').request(tvOptions);
        }

        var siblings = node.get('siblings');

        siblings.forEach(function (item) {
            item.set('shown', false);
        }, this);
        node.set('shown', true);

        // App.log('[menu focus]', node, tvOptions);

        switch (node.get('selector')) {
            case '#search':
            break;
            default:
            break;
        }

        return true;
    },

    keyDown: function (key) {
        if (key.which || key.keyCode) {
            var event = key;
            key = event.which || event.keyCode;
        }

        var index = this.index(),
            siblings = this.get('allSiblings'), // .get('children'), // TODO: move to method?
            length = siblings.length;

        if (App.MainMenuLength && this.get('parent') === App.tree) {
            length = App.MainMenuLength;
        }

        // App.log('[RulerNode.keydown]', key, 'index', index, 'length', length);

        switch (key) {
        case ENV.KEY.RIGHT:
            if (index < length) {
                this.focus(siblings[index + 1]);
            }
            break;
        case ENV.KEY.LEFT:
            if (index > 0) {
                this.focus(siblings[index - 1]);
            }
            break;
        case ENV.KEY.DOWN:
            this.down();
            break;
        case ENV.KEY.UP:
            this.up();
            break;
        case ENV.KEY.ENTER:
            this.focus();
            break;
        default:
            return false;
            break;
        }
        return true;
    },

    rulerMove: function (left, top) {
        var activeIndex = this.get('activeChild').index();
        this.get('children').forEach(function (item, index) {
            if (Em.Object.detectInstance(item) &&
                item.get('hidden') !== index < activeIndex || index > activeIndex + 3
            ) {
                item.set('hidden', index < activeIndex || index > activeIndex + 3);
            } else if (!Em.Object.detectInstance(item)) {
                App.log('[rulerMove] null item', item, index, this);
            }
        });
        // left = left || - this.get('activeChild').$().position().left;
        // this.get('childrenController').$().css({
        //     left: left,
        // })
    }
});

App.Tree.TVWeekdayNode = App.Tree.RulerNode.extend({
    text: null,
    day: null,
    shown: false,
    highlighted: false,
    focus: function (node) {
        this._super(node);

        node = node || this;
        node.get('siblings').forEach(function (item) {
            item.set('highlighted', false);
        });
        node.set('highlighted', true);
    },
    // rulerMove: function (left, top) {
    //     left = left || - this.get('activeChild').$().position().left;
    //     this.get('childrenController').$().css({
    //         left: left,
    //     })
    // },
});

App.Tree.TVChannelNode = App.Tree.RulerNode.extend({
    text: null,
    // arrowUpShown: false,
    // arrowDownShown: false,
    rulerMove: function (left, top) {
        this.get('parent').rulerMove();
    },
    focus: function (node) {
        this._super(node);

        node = node || this;
        node.rulerMove();
        var allSiblingsLength = node.get('allSiblings').length;
        if (App.Tree.TVChannelNode.detectInstance(node) && allSiblingsLength && allSiblingsLength - node.index() < 3) {
            var siblings = node.get('allSiblings');
            var tvOptions = {
                tv_day: node.get('parent').get('day'),
                tv_channel_list: siblings[siblings.length - 1]
                    .get('children').objectAt(0).get('data').id_tv // last tv channel ID
            };
            App.log('[App.Tree.TVChannelNode] tv_channel_list', tvOptions.tv_channel_list);
            node.get('model').request(tvOptions);
        }
    },
    model: Em.computed(function () {
        return this.get('parent').get('model');
    }).property('parent'),
});

App.Tree.DetailsController = App.Tree.ItemNode.extend({
    selector: '#details',
    view: null,
    focus: function (node) {

    },
    // viewShownChanged: function () {
    //     if (!this.get('view').get('shown')) {
    //         this.get('view').$().find('.synopsis').css('top', 0);
    //     }
    // }.observes('view.shown'), // to properly rewrite in [platform].js
    currentChanged: function () {
        var current = App.tree.get('current');
        if (App.Tree.RulerItemNode.detectInstance(current)) {
            if (App.SearchRulerController === current.get('parent') ||
                App.Tree.TVChannelNode.detectInstance(current.get('parent'))
            ) {
                this.$().css('width', '');
            } else {
                this.$().css('width', $('body').innerWidth() -  64 - current.$().innerWidth());
                // this.$().find('.poster').css('width', current.$().find('.poster').outerWidth());
            }
        }
    }.observes('App.tree.current')
});

App.Tree.TrailerController = App.Tree.ControlsNode.extend({
    id: 'trailer',
    text: 'ПОСМОТРЕТЬ ТРЕЙЛЕР',
    forceHide: false,
    selector: '#controls .trailer',
    trailerView: null,
    shown: false,
    shownShouldChange: function () {
        this.set('shown',
            Em.get('App.tree.current.data.trailer.src') && !this.get('trailerView.shown'));
    }.observes('App.tree.current', 'trailerView.shown'),
    action: function (show) {
        if (typeof show === 'undefined') {
            show = true;
        }
        this.set('trailerView.shown', show && this.get('shown'));
    },
    focus: function (event) {
        this.action();
    }
});

App.KeyController = Em.Object.extend({
    keyPress: function (event) {
        if (typeof event.stopPropagation === 'function') {
            event.stopPropagation();
        }
        App.log('[keypress] ', event.which || event.keyCode, event);
    },
    lastKeyDownTimestamp: null,
    keyDown: function (event) {
        if (!App.tree.get('current')) {
            return false;
        }

        // preventing intensive keypresses
        var lastTimespamp = this.get('lastKeyDownTimestamp'),
            timestamp = $.now();
        if (!lastTimespamp || timestamp - lastTimespamp > 100) {
            this.set('lastKeyDownTimestamp', timestamp);
        } else {
            return false;
        }

        App.tree.set('mouse', false);

        var key = event.which || event.keyCode; // Opera
        if (typeof event.stopPropagation === 'function') {
            event.stopPropagation();
        }

        // if (App.DetailsView.get('shown')) {
        //     App.DetailsController.keyDown(key);
        // }

        if (App.TrailerView.get('shown') /*&& !App.TrailerView.get('autohideTimeout')*/) {
            App.TrailerView.toolbar(event);
            return;
        }

        if (App.Tree.RulerItemNode.detectInstance(App.tree.get('current'))) { // [TODO: not really] ruler controller
            App.tree.get('current').keyDown(key);
        } else if (App.SearchRulerController === App.tree.get('current').get('parent')) {
            App.tree.get('current').keyDown(key);
        } else if (App.Tree.RulerNode.detectInstance(App.tree.get('current'))) {
            App.tree.get('current').keyDown(key);
        } else if (App.keyboard === App.tree.get('current')) {
            App.tree.get('current').keyDown(key);
        } else {
            App.log('[KeyController.keydown] ' + (event.which || event.keyCode));
            switch (key) {
            case ENV.KEY.UP:
                App.log(key);
                break;
            default:
                break;
            }
        }
    },
    keyUp: function (event) {

    }
});

App.MouseController = Em.Object.extend({
    move: function (event) {
        if (Em.get('App.TrailerController.trailerView.shown')) {
            App.TrailerView.toolbar(event);
            return;
        }
    },
    click: function (event) {
        // TODO: in fact, it's a workaround for the bug: find() works inproperly
        if (event.target === document.querySelector('body')) {
            return false;
        }

        if ([].indexOf.call(document.querySelectorAll('.mousenav, .mousenav *'), event.target) !== -1) {
            var current = App.tree.get('current');
            if (App.Tree.RulerNode.detectInstance(current)) {
                current.down();
            } else if (current === App.keyboard) {
                current.get('parent').get('parent').down();
            }
            App.MouseNavController.action(
                (event.target.parentNode.classList.contains('left') || event.target.classList.contains('left')) ? 'left' :
                (event.target.parentNode.classList.contains('right') || event.target.classList.contains('right')) ? 'right' :
                (event.target.parentNode.classList.contains('up') || event.target.classList.contains('up')) ? 'up' :
                'down');
            return;
        }

        if (App.TrailerView.get('shown') && !App.TrailerView.get('autohideTimeout')) {
            App.TrailerView.toolbar(event);
            return;
        }

        var targetNode = App.tree.find(event.target);
        App.log('[App.MouseController.click]', event, 'mouse:', App.tree.get('mouse'), 'targetNode', targetNode);
        var doubleFocus = App.tree.get('mouse') && targetNode !== App.tree.get('current'); // && !App.Tree.ControlsNode.detectInstance(App.tree.get('current'));

        if (targetNode && targetNode !== App.tree && targetNode.focus) {
            targetNode.focus();
            if (doubleFocus) {
                targetNode.focus();
            }
        }
    },
    focus: function (event) {
        var target = App.tree ? App.tree.find(event.target) : null;
        App.log('focus', event);

        if (target && target.focus) {
            target.focus();
        }
    }
});



/**
 * Views
 **/

App.start = function () {
    App.RulerViewClass = Em.View.extend({
        willInsertElement: function () {
            // TODO: this is ugly :(
            var initNode = App.NowRulerController;
            function initFocus () {
                if (initNode.$().length) {
                    $('body').addClass('no-splash');
                    initNode.focus();
                } else {
                    setTimeout(initFocus, 100);
                }
            }

            initFocus();
        }
    });

    App.DetailsViewClass = Em.View.extend({
        classNameBindings: [ 'realShown:shown', 'withPoster:with-poster' ],
        title: null,
        shown: false,
        realShown: false,
        withPoster: false,
        awaitPercent: false,
        rating: 0,
        tvName: null,
        tvTime: null,
        poster: null,
        statusLeftColor: 'gray', // color of 'Loading'/video progress
        scrollBarHeight: 0,
        scrollBarTop: 0,
        scrollBarStyle: function () {
            if (App.NavigationVerticalController &&
                App.NavigationVerticalController.get('shown')
            ) {
                return 'background-size: 10px {height}%; background-position: right {top}px;'
                    .replace('{height}', this.get('scrollBarHeight'))
                    .replace('{top}', this.get('scrollBarTop'));
            } else {
                return '';
            }
        }.property('scrollBarHeight', 'scrollBarTop'),
        shownChanged: function () {
            if (this.get('shown')) {
                var current = App.tree.get('current'),
                    data = current.get('data');

                if (!data) {
                    this.set('shown', false);
                    return false;
                }

                this.set('director', /*stripHTML*/data.directorName ?
                    (data.directorName.split(/<br\ ?\/?>/).slice(0, 3).join(',<br />') || null) : // TODO: for now there's always 1 director
                    null);

                if (data.actors && data.actors.length) {
                    this.set('actors', data.actors.slice(0, 5));
                }
                this.set('title', data.title || data.rus || data.name);
                if (data.percent) {
                    this.set('awaitPercent', Math.round(data.percent) + '%');
                    this.set('rating', null);
                } else {
                    this.set('awaitPercent', null);
                    this.set('rating', data.rating); // precision 0.1
                }

                if (data.tv_name) {
                    this.set('tvName', data.tv_name);
                    this.set('tvTime', moment(data.day + 'T' + data.seans_t).format('dddd, HH:mm')); // TODO: moment() localization
                }

                this.set('withPoster', /*true*/ App.SearchRulerController === current.get('parent') || App.Tree.TVChannelNode.detectInstance(current.get('parent')));
                if (this.get('withPoster') && data.image && !data.no_poster) {
                    this.set('poster', 'background-image: url(' + data.image + ')');
                } else {
                    this.set('poster', 'background-image: url(http://st.kinopoisk.ru/images/api_tv/noposter.gif)');
                }

                // if (data.back_url) {
                //     $('body').css('backgroundImage', 'url(' + data.back_url + ')');
                //     App.DetailsController.$().css('background', 'transparent');
                // }
            } else {
                this.set('actors', null);
                this.set('title', null);
                this.set('awaitPercent', null);
                this.set('tvName', null);
                this.set('rating', null);
                this.set('withPoster', false);
                this.set('poster', null);
                this.set('director', null);

                // $('body').css('backgroundImage', 'none');
                // App.DetailsController.$().css('background', 'black');
            }
            this.set('realShown', this.get('shown'));
        }.observes('shown'),
        // willRerender: function () {
        //     console.log('[willRerender]', arguments);
        // },
        willInsertElement: function () {
            this.addObserver('shown', App.Model.loadSynopsis.bind(this));
            App.Preloader.add([
                'http://st.kinopoisk.ru/images/api_tv/details-movie-desc.png',
                'http://st.kinopoisk.ru/images/api_tv/noposter.gif'
            ]);
        }
    });

    App.ControlsViewClass = Em.View.extend({
        size: 'small',
        // border: Em.computed(function () {
        //     return
        // }).property('App.TrailerView.shown', 'App.TrailerView.toolbar'),
        shown: function () {
            App.log('[App.ControlsViewClass] shown',
                Em.get('App.TrailerController.trailerView.shown'), Em.get('App.TrailerController.trailerView.autohideTimeout'));
            return !Em.get('App.TrailerController.trailerView.shown') || Em.get('App.TrailerController.trailerView.autohideTimeout');
        }.property('App.TrailerController.trailerView.shown', 'App.TrailerController.trailerView.autohideTimeout')
    });

    App.StatusViewClass = Em.View.extend({
        classNameBindings: ['shown'],
        shown: false,
        text: '',
        willInsertElement: function () {
            if (typeof ENV.USER_AGENT === 'undefined') {
                $.get(ENV.REQUEST_HANDLER + '?useragent').done(function (r) {
                    ENV.USER_AGENT = r;
                    App.log(ENV.USER_AGENT);
                });
            } else {
                App.log(ENV.USER_AGENT);
            }
        }
    });

    App.TrailerViewClass = Em.View.extend({
        willInsertElement: function () {
            App.PauseController.reopen({
                shownChanged: function () {
                    App.log('[App.PauseController shownChanged]');
                    this.set('shown', App.TrailerView.get('shown'));
                }.observes('App.TrailerController.trailerView.shown')
            });
            // Em.bind(App.PauseController, "shown", "App.TrailerController.trailerView.shown");
        },
        autohideTimeout: null,
        toolbar: function (event) {
            // hide toolbar
            if (typeof event === 'undefined' &&
                !(App.DetailsView.get('statusLeftShown') &&
                    App.DetailsView.get('statusLeftText') == ENV.LANG.RU.LOADING_TRAILER)
            ) {
                // App.PauseController.set('shown', false);
                // App.BackController.set('shown', false);
                clearTimeout(this.get('autohideTimeout'));
                this.set('autohideTimeout', null);
            } else /*if (!this.get('autohideTimeout'))*/ {
                // if (!App.PauseController.get('shown')) {
                //     App.PauseController.set('shown', true);
                // }
                // if (!App.BackController.get('shown')) {
                //     App.BackController.set('shown', true);
                // }
                clearTimeout(this.get('autohideTimeout'));
                this.set('autohideTimeout', setTimeout(this.toolbar.bind(this), 3000));
            }
        },
        shownChanged: function () {
            if (this.get('shown')) {
                this.set('autohideTimeout', setTimeout(this.toolbar.bind(this), 3000));
            } else {
                clearTimeout(this.get('autohideTimeout'))
                this.set('autohideTimeout', null);
            }
        }.observes('shown')
    });

    App.MenuViewClass = Em.View.extend({
        willInsertElement: function () {
            App.Preloader.add('http://st.kinopoisk.ru/images/api_tv/menu.png');
        },
        // willRerender
        // becameVisible
    });

    App.tree = App.Tree.RootNode.create({
        id: 'root',
        selector: 'body',
        mouse: false
    });

    // Below set() is used to trigger observer

    App.NowRulerController = App.Tree.RulerNode.create({
        id: 'now',
        selector: '.nav li:nth-child(1)'
    }).set('childrenController', Em.ArrayController.create({
        selector: '#now .ruler',
        _model: App.Model.FilmList.create({
            type: 'now'
        })
    }));

    App.SoonRulerController = App.Tree.RulerNode.create({
        id: 'soon',
        selector: '.nav li:nth-child(2)'
    }).set('childrenController', Em.ArrayController.create({
        selector: '#soon .ruler',
        _model: App.Model.FilmList.create({
            type: 'soon'
        })
    }));

    App.TVRulerController = App.Tree.RulerNode.create({
        id: 'tv',
        selector: '.nav li:nth-child(3)',
        isRequestingChanged: function() {
            App.LoaderController.set('shown', this.get('isRequesting'));
        }.observes('isRequesting'),
        shownChanged: function () {
            if (this.get('activeChild')) {
                this.get('activeChild').set('shown', this.get('shown'));
            }
        }.observes('shown')
    }).set('childrenController', Em.ArrayController.create({
        selector: '#weekdays',
        _model: App.Model.TVList.create()
    }));

    App.SearchRulerControllerClass = App.Tree.RulerNode.extend({
        keyDown: function (key) {
            // TODO: this will kill arrow keys - input navigation. Probably
            this._super(key);
            // if (this._super(event)) {
            //     return true;
            // }
            App.log('[SearchRulerController.keyDown] ' + key);
            // var mouseEvent = event instanceof window.MouseEvent;
            // if (mouseEvent ||
            //     document.querySelector('#search-input input') === event.target) {
            //     var key = mouseEvent ?
            //         $(event.target).attr('name').match(/keyCode_(\d+)/)[1] :
            //         (event.which || event.keyCode);
            //     App.log('[search keydown] key', key, event.target, mouseEvent);
            //     this.get('model').autocomplete({ target: $('#search-input input') });
            // }

            // TODO: mouse events trigger 2 timer = below fix does not work
            // if (mouseEvent) {
            //     event.stopPropagation();
            // }
        },
        searchFieldShown: false,

        // TODO: below doesn't work?
        // searchFieldShownChanged: function () {
        //     if (this.get('searchFieldShown')) {
        //         this.$().find('input[type="text"]').trigger('focus');
        //     }
        // }.observes('searchFieldShown'),

        edit: function (event) {
            App.log('[input edit]', event);
        }
    });
    App.SearchRulerController = App.SearchRulerControllerClass.create({
        more: false,
        id: 'search',
        selector: '.nav li:nth-child(4)',
        placeholderText: 'Название фильма',
        keyboard: null,
    }).set('childrenController', Em.ArrayController.create({
        selector: '#search .ruler',
        _model: App.Model.FilmSearch.create()
    }));

    App.RefreshControllerClass = App.Tree.ControlsNode.extend({
        action: function () {
            if (this.get('shown')) {
                window.location.reload();
            }
        }
    });
    App.RefreshController = App.RefreshControllerClass.create({
        id: 'refresh',
        text: 'ОБНОВИТЬ',
        shown: true,
        selector: '#controls .refresh',
    });

    App.ExitControllerClass = App.Tree.ControlsNode.extend({
        action: function () {
            return false;
        }
    });
    App.ExitController = App.ExitControllerClass.create({
        id: 'exit',
        text: 'ЗАКРЫТЬ',
        shown: true,
        selector: '#controls .exit',
    });

    App.BackControllerClass = App.Tree.ControlsNode.extend({
        id: 'back',
        text: 'НАЗАД',
        selector: '#controls .back',
        action: function () { // TODO: focus?
            // TODO: implement full history navigation or use Ember routes
            // if (!this.get('shown')) {
            //     return false;
            // }

            // if (App.tree.get('current') !== this) {
            //     this.focus();
            // }

            if (App.TrailerView.get('shown')) {
                App.TrailerController.action(false);
                // App.tree.get('current').focus();
                if (App.Tree.ControlsNode.detectInstance(App.tree.get('current'))) {
                    App.tree.get('previous').focus();
                }
            } else if (App.DetailsView.get('shown')) {
                // App.tree.get('current').focus();
                // App.DetailsView.set('shown', false);
                if (App.Tree.ControlsNode.detectInstance(App.tree.get('current'))) {
                    App.tree.get('previous').focus();
                } else {
                    App.DetailsView.set('shown', false);
                }
            } else if ([App.NowRulerController,
                        App.SoonRulerController,
                        App.TVRulerController,
                        App.SearchRulerController]
                        .indexOf(App.tree.get('current')) != -1
            ) {
                if (App.tree.get('current') !== App.NowRulerController) {
                    App.NowRulerController.focus();
                } else {
                    App.ExitController.action();
                }
            } else {
                App.NowRulerController.focus();
            }
        },
        shown: function () {
            var result =
                App.tree.get('current') !== App.NowRulerController;
                // (App.TrailerView && App.TrailerView.get('shown')) ||
                // (App.DetailsView && App.DetailsView.get('shown')) ||
                // (App.SearchRulerController.get('shown') && !App.Tree.RulerNode.detectInstance(App.tree.get('current')));
            return result;
        }.property('App.tree.current')
        // }.property('App.TrailerController.trailerView.shown', 'App.DetailsView.shown', 'App.tree.current')
    });
    App.BackController = App.BackControllerClass.create();

    App.PauseControllerClass = App.Tree.ControlsNode.extend({
        shown: false,
        text: 'ПАУЗА',
        action: function () { // TODO: focus?
            if (this.get('shown')) {
                if (this.get('text') === 'ПАУЗА') {
                    App.TrailerView.play(0);
                    this.set('text', 'ПУСК');
                    this.set('status', 'play');
                } else {
                    App.TrailerView.play(1);
                    this.set('text', 'ПАУЗА');
                    this.set('status', 'pause');
                }
            }
        }
    });
    App.PauseController = App.PauseControllerClass.create({
        id: 'pause',
        status: 'pause',
        selector: '#controls .pause',
    });

    App.LoaderController = App.Tree.ControlsNode.create({
        shown: false,
        text: 'Загрузка...'
    });

    App.MouseNavControllerClass = App.Tree.ControlsNode.extend({
        selector: '.mousenav',
        _shownRandom: null, // TODO: this is fcking insane
        shown: function () {
            var shown = {},
                current = App.tree.get('current');
            if (!current || /*App.BackController.get('shown') ??? ||*/ current === App.tree) {
                return shown;
            }

            // TV program list
            if (App.Tree.TVChannelNode.detectInstance(current.get('parent'))) {
                var index = current.index();
                shown.up = current.get('siblings').filter(function (item) {
                    return item && item.get('hidden') && item.index() < index;
                }).length > 0;
                shown.down = current.get('siblings').filter(function (item) {
                    return item && item.get('hidden') && item.index() > index;
                }).length > 0;

                if (App.tree.get('mouse')) {
                    var parent = current.get('parent');
                    index = parent.index();
                    shown.left = parent.get('siblings').filter(function (item) {
                        return item && item.get('hidden') && item.index() < index;
                    }).length > 0; // index > 0;
                    shown.right = parent.get('parent').get('more') || parent.get('siblings').filter(function (item) {
                        return item && item.get('hidden') && item.index() > index;
                    }).length > 0;
                }
            } else if (App.tree.get('mouse')) {
                if (App.Tree.RulerNode.detectInstance(current)) {
                    // App.log(' parent ok ' + current.get('children').length);
                    current = current.get('activeChild');
                } else if (current === App.keyboard) {
                    // TODO: BUG: this returns Back button if triggered at the second search request
                    // current = App.tree.get('activeChild').get('activeChild');
                    current = App.SearchRulerController.get('activeChild');
                }
                if (App.Tree.RulerItemNode.detectInstance(current)) {
                    var index = current.index();
                    shown.left = current.get('siblings').filter(function (item) {
                        return item && item.get('hidden') && item.index() < index;
                    }).length > 0; // index > 0;
                    shown.right = current.get('parent').get('more') || current.get('siblings').filter(function (item) {
                        return item && item.get('hidden') && item.index() > index;
                    }).length > 0;
                }
            }

            return shown;
        }.property('App.tree.mouse', 'App.BackController.shown', 'App.tree.current', '_shownRandom'),
        update: function () {
            this.set('_shownRandom', Math.random());
        },
        action: function (side) {
            switch (side) {
                case 'left':
                    App.tree.get('current').keyDown(ENV.KEY.LEFT);
                break;
                case 'right':
                    App.tree.get('current').keyDown(ENV.KEY.RIGHT);
                break;
                case 'up':
                    App.tree.get('current').keyDown(ENV.KEY.UP);
                break;
                case 'down':
                    App.tree.get('current').keyDown(ENV.KEY.DOWN);
                break;
            }

        }
    });
    App.MouseNavController = App.MouseNavControllerClass.create();

    // TODO: create 'focused' in templates, remove initFocus(), then uncomment
    App.tree.set('children', [
        App.NowRulerController,
        App.SoonRulerController,
        App.TVRulerController,
        App.SearchRulerController,

        App.BackController,
        // App.TrailerController,
        App.RefreshController,
        App.ExitController,
        App.PauseController,
        App.MouseNavController
    ]);

    App.mouseController = App.MouseController.create();
};

function stripHTML(html) {
    if (html) {
        var tmp = document.createElement("DIV");
        tmp.innerHTML = html.replace(/\&amp\;nbsp\;/ig, ' ');
        return tmp.textContent||tmp.innerText;
    } else {
        return false;
    }
}
